@@ -14,6 +14,7 @@ import graphviz as gv
 import UserDict
 import sys
 import threading
+import warnings
 
 class PipeReader(threading.Thread):
     """Read and write pipes using threads.
@@ -48,27 +49,93 @@ class AGraph(object):
 
       G=AGraph("file.dot")   
 
-    
-    """
-    def __init__(self, file=None, name=None,
-                 data=None, strict=True, directed=False,
-                 handle=None):
+    Graphviz graph keyword parameters are processed so you may add
+    them like
 
-        if handle is None:
-            # the graph pointer (handle)
-            self.handle=gv.agraphnew(name,strict,directed)  
+    >>> G=AGraph(landscape='true',ranksep='0.1')
+
+    or alternatively
+
+    >>> G=AGraph()
+    >>> G.graph_attr.update(landscape='true',ranksep='0.1')
+
+    and
+
+    >>> G.node_attr.update(color='red')
+    >>> G.edge_attr.update(len='2.0',color='blue')
+
+    See http://www.graphviz.org/doc/info/attrs.html
+    for a list of attributes.
+    
+
+    Keyword parameters:
+
+    thing is a generic input type (filename, string, handle to pointer, 
+    dictionary of dictionaries).  An attempt is made to automaticaly
+    detect the type so you may write for example:
+
+    >>> d={'1': {'2': None}, '2': {'1': None, '3': None}, '3': {'2': None}}
+    >>> A=AGraph(d)
+    >>> s=AGraph.to_string()
+    >>> B=AGraph(s)
+    >>> h=AGraph.handle
+    >>> C=AGraph(h)
+
+    name is a string name for the graph        
+
+    strict=True|False (True for simple graphs)
+
+    directed=True|False
+
+    data is a dictionary of dictionaries or dictionary of lists
+    representing nodes or edges to load into intial graph
+
+    string is a string containing a dot format graph
+
+    handle is a Swig pointer to an agraph_t data structure
+
+    """
+    def __init__(self, thing=None, file=None, name=None,
+                 data=None, string=None, strict=True, directed=False,
+                 handle=None,**attr):
+
+        # atttempt to guess input type
+        if isinstance(thing,dict):
+            data=thing # a dictionary of dictionaries (or lists)
+        if hasattr(thing,'own'): # a Swig pointer - graph handle
+            handle=thing
+        if self._is_string_like(thing): 
+            if (thing.startswith("strict") or 
+                thing.startswith("digraph") or 
+                thing.startswith("graph")):
+               string=thing # this is a dot format graph in a string
+            else:
+                file=thing  # assume this is a file name
+
+        # input type guessed or specified - now init graph                
+        if handle is None:  # the graph pointer (handle)
+            self.handle=gv.agraphnew(name,strict,directed) # new graph
         else:
-            self.handle=handle
+            self.handle=handle # use pointer to exisiting graph
+
+        # read from filename
         if file is not None:
             self.read(file)
 
-        if data is not None: # dict of dicts or dict of lists
+        # load from dict of dicts or dict of lists
+        if data is not None:
             for node in data:
                 for nbr in data[node]:
                     self.add_edge(node,nbr)
             self.add_nodes_from(data.keys())        
 
+        # load from string
+        if string is not None:
+            self.from_string(string)
+
+        # assign any attributes specified through keywords
         self.graph_attr=Attribute(self.handle,0) # default graph attributes
+        self.graph_attr.update(attr) # apply attributes passed to init
         self.node_attr=Attribute(self.handle,1)  # default node attributes
         self.edge_attr=Attribute(self.handle,2)  # default edge attribtes
 
@@ -114,7 +181,7 @@ class AGraph(object):
 #    def __setitem__(self,u,v):
 #        self.add_edge(u,v)
 
-    def add_node(self, n):
+    def add_node(self, n, **attr):
         """Add a single node n to the graph.
 
         If n is not a string, conversion to a string will be attempted.
@@ -129,14 +196,25 @@ class AGraph(object):
         >>> G.nodes()
         ['a', '1']
         
+        Attributes can be added to nodes on creation
+
+        >>> G.add_node(2,color='red')
+
+        See http://www.graphviz.org/doc/info/attrs.html
+        for a list of attributes.
+
+        Anonymous Graphviz nodes are currently not implemented.
+
         """
         if not self._is_string_like(n):  n=str(n)
         try:
             nh=gv.agnode(self.handle,n,_Action.find)
         except KeyError:
             nh=gv.agnode(self.handle,n,_Action.create)
+            nh=Node(self,n)
+            nh.attr.update(**attr)
 
-    def add_nodes_from(self, nbunch):
+    def add_nodes_from(self, nbunch, **attr):
         """Add nodes to graph from a container nbunch.
 
         nbunch can be any iterable container such as a list or dictionary
@@ -147,9 +225,14 @@ class AGraph(object):
         >>> sorted(G.nodes())
         ['1', 'a', 'b', 'spam']
         
+
+        Attributes can be added to nodes on creation
+
+        >>> G.add_nodes_from(nlist, color='red') # set all nodes in nlist red
+
         """
         for n in nbunch:
-            self.add_node(n)
+            self.add_node(n,**attr)
 
     def delete_node(self,n):
         """Delete the single node n from graph.
@@ -232,7 +315,7 @@ class AGraph(object):
         """
         return Node(self,n)
 
-    def add_edge(self,u,v=None,key=None):  
+    def add_edge(self,u,v=None,key=None,**attr):  
         """Add a single edge between nodes u and v to the graph.
 
         If u and v are not nodes in they graph they will added.
@@ -256,6 +339,13 @@ class AGraph(object):
         >>> sorted(G.edges())
         [('a', 'b', 'first'), ('a', 'b', 'second')]
 
+        Attributes can be added when edges are created
+
+        >>> G.add_edge('a','b',color='green')
+
+        See http://www.graphviz.org/doc/info/attrs.html
+        for a list of attributes.
+
         """
         if v is None: (u,v)=u  # no v given, assume u is an edge tuple
         try:
@@ -270,10 +360,12 @@ class AGraph(object):
             vh=Node(self,v).get_handle()
         try:
             eh=gv.agedge(uh,vh,key,_Action.create)
+            eh=Edge(self,u,v,key,eh)
+            eh.attr.update(**attr)
         except KeyError:
             return None # silent failure for strict graph, already added
 
-    def add_edges_from(self, ebunch):  
+    def add_edges_from(self, ebunch, **attr):  
         """Add nodes to graph from a container ebunch.
 
         ebunch is a container of edges such as a list or dictionary.
@@ -281,9 +373,14 @@ class AGraph(object):
         >>> G=AGraph()
         >>> elist=[('a','b'),('b','c')]
         >>> G.add_edges_from(elist)
+
+        Attributes can be added when edges are created
+
+        >>> G.add_edges_from(elist, color='green')
+
         """
         for e in ebunch:
-            self.add_edge(e)
+            self.add_edge(e,**attr)
 
     def get_edge(self, u, v, key=None):
         """Return an edge object (Edge) corresponding to edge (u,v).
@@ -352,8 +449,7 @@ class AGraph(object):
                 while eh is not None:
                     (s,t)=(gv.agtail(eh),gv.aghead(eh))
                     (u,v)=(gv.agnameof(s),gv.agnameof(t))
-                    key=gv.agnameof(eh)
-                    yield Edge(self,u,v,key)
+                    yield Edge(self,u,v,key=gv.agnameof(eh),eh=eh)
                     eh=gv.agnxtout(eh)
                 nh=gv.agnxtnode(nh)
         elif nbunch in self: # if nbunch is a single node 
@@ -363,7 +459,7 @@ class AGraph(object):
             while eh is not None:
                 (s,t)=(gv.agtail(eh),gv.aghead(eh))
                 (u,v)=(gv.agnameof(s),gv.agnameof(t))
-                yield Edge(self,u,v,gv.agnameof(eh))
+                yield Edge(self,u,v,key=gv.agnameof(eh),eh=eh)
                 eh=gv.agnxtedge(eh,nh)
         else:                # if nbunch is a sequence of nodes
             try: bunch=[n for n in nbunch if n in self]
@@ -379,7 +475,7 @@ class AGraph(object):
                 while eh is not None:
                     (s,t)=(gv.agtail(eh),gv.aghead(eh))
                     (u,v)=(gv.agnameof(s),gv.agnameof(t))
-                    yield Edge(self,u,v,gv.agnameof(eh))
+                    yield Edge(self,u,v,key=gv.agnameof(eh),eh=eh)
                     eh=gv.agnxtout(eh)
         raise StopIteration
  
@@ -448,7 +544,7 @@ class AGraph(object):
                 while eh is not None:
                     (s,t)=(gv.agtail(eh),gv.aghead(eh))
                     (u,v)=(gv.agnameof(s),gv.agnameof(t))
-                    yield Edge(self,u,v,gv.agnameof(eh))
+                    yield Edge(self,u,v,key=gv.agnameof(eh),eh=eh)
                     eh=gv.agnxtout(eh)
                 nh=gv.agnxtnode(nh)
         elif nbunch in self: # if nbunch is a single node 
@@ -458,7 +554,7 @@ class AGraph(object):
             while eh is not None:
                 (s,t)=(gv.agtail(eh),gv.aghead(eh))
                 (u,v)=(gv.agnameof(s),gv.agnameof(t))
-                yield Edge(self,u,v,gv.agnameof(eh))
+                yield Edge(self,u,v,key=gv.agnameof(eh),eh=eh)
                 eh=gv.agnxtout(eh)
         else:                # if nbunch is a sequence of nodes
             try: bunch=[n for n in nbunch if n in self]
@@ -474,7 +570,7 @@ class AGraph(object):
                 while eh is not None:
                     (s,t)=(gv.agtail(eh),gv.aghead(eh))
                     (u,v)=(gv.agnameof(s),gv.agnameof(t))
-                    yield Edge(self,u,v,gv.agnameof(eh))
+                    yield Edge(self,u,v,key=gv.agnameof(eh),eh=eh)
                     eh=gv.agnxtout(eh)
         raise StopIteration
  
@@ -494,7 +590,7 @@ class AGraph(object):
                 while eh is not None:
                     (s,t)=(gv.agtail(eh),gv.aghead(eh))
                     (u,v)=(gv.agnameof(s),gv.agnameof(t))
-                    yield Edge(self,u,v,gv.agnameof(eh))
+                    yield Edge(self,u,v,key=gv.agnameof(eh),eh=eh)
                     eh=gv.agnxtin(eh)
                 nh=gv.agnxtnode(nh)
         elif nbunch in self: # if nbunch is a single node 
@@ -504,7 +600,7 @@ class AGraph(object):
             while eh is not None:
                 (s,t)=(gv.agtail(eh),gv.aghead(eh))
                 (u,v)=(gv.agnameof(s),gv.agnameof(t))
-                yield Edge(self,u,v,gv.agnameof(eh))
+                yield Edge(self,u,v,key=gv.agnameof(eh),eh=eh)
                 eh=gv.agnxtin(eh)
         else:                # if nbunch is a sequence of nodes
             try: bunch=[n for n in nbunch if n in self]
@@ -520,7 +616,7 @@ class AGraph(object):
                 while eh is not None:
                     (s,t)=(gv.agtail(eh),gv.aghead(eh))
                     (u,v)=(gv.agnameof(s),gv.agnameof(t))
-                    yield Edge(self,u,v,gv.agnameof(eh))
+                    yield Edge(self,u,v,key=gv.agnameof(key),eh=eh)
                     eh=gv.agnxtin(eh)
         raise StopIteration
 
@@ -679,8 +775,18 @@ class AGraph(object):
         if self.is_directed():
             # new empty DiGraph
             H=self.__class__(strict=self.is_strict(),directed=True) 
-            H.add_nodes_from(self)
-            H.add_edges_from([(v,u) for (u,v) in self.edges_iter()])
+            H.graph_attr.update(self.graph_attr)
+            H.node_attr.update(self.node_attr)
+            H.edge_attr.update(self.edge_attr)
+            for n in self.nodes():
+                H.add_node(n)
+                new_n=Node(H,n)
+                new_n.attr.update(n.attr)
+            for e in self.edges():
+                (u,v)=e
+                H.add_edge(v,u)
+                uv=H.get_edge(v,u)
+                uv.attr.update(e.attr)
             return H
         else:
             return self
@@ -715,9 +821,9 @@ class AGraph(object):
 
     def clear(self):
         """Remove all nodes, edges, and attributes from the graph."""
-        for a in self.edge_attr:  del self.edge_attr[a]
-        for a in self.node_attr:  del self.node_attr[a]
-        for a in self.graph_attr: del self.graph_attr[a]
+        self.edge_attr.clear()
+        self.node_attr.clear()
+        self.graph_attr.clear()
         self.delete_edges_from(self.edges())
         self.delete_nodes_from(self.nodes())
 
@@ -770,7 +876,6 @@ class AGraph(object):
         """
         handle=gv.agsubg(self.handle,name,_Action.create)
         H=self.__class__(strict=self.is_strict(),directed=self.is_directed(),handle=handle,name=name)
-
         if nbunch is None: return H
         # add induced subgraph on nodes in nbunch
         bunch=self.prepare_nbunch(nbunch)
@@ -778,8 +883,10 @@ class AGraph(object):
         for (u,v) in self.edges():
             if u in H and v in H: 
                 H.add_edge(u,v)
+
         return H
 
+
     def delete_subgraph(self, name):
         """Delete subgraph with given name."""  
         handle=gv.agsubg(self.handle,name,_Action.find)
@@ -860,20 +967,18 @@ class AGraph(object):
             return self.copy()
         else:
             U=AGraph(strict=self.is_strict())
-            for (k,v) in self.edge_attr.iteritems(): U.edge_attr[k]=v
-            for (k,v) in self.node_attr.iteritems(): U.node_attr[k]=v
-            for (k,v) in self.graph_attr.iteritems(): U.graph_attr[k]=v
+            U.graph_attr.update(self.graph_attr)
+            U.node_attr.update(self.node_attr)
+            U.edge_attr.update(self.edge_attr)
             for n in self.nodes():
                 U.add_node(n)
                 new_n=Node(U,n)
-                for (k,v) in n.attr.iteritems():
-                    new_n.attr[k]=v
+                new_n.attr.update(n.attr)
             for e in self.edges():
                 (u,v)=e
                 U.add_edge(u,v)
                 uv=U.get_edge(u,v)
-                for (k,v) in e.attr.iteritems():
-                    uv.attr[k]=v
+                uv.attr.update(e.attr)
             return U
 
 
@@ -885,23 +990,22 @@ class AGraph(object):
         """
         if self.is_undirected():
             D=AGraph(strict=self.is_strict(),directed=True)
-            for (k,v) in self.edge_attr.iteritems(): D.edge_attr[k]=v
-            for (k,v) in self.node_attr.iteritems(): D.node_attr[k]=v
-            for (k,v) in self.graph_attr.iteritems(): D.graph_attr[k]=v
+            D.graph_attr.update(self.graph_attr)
+            D.node_attr.update(self.node_attr)
+            D.edge_attr.update(self.edge_attr)
             for n in self.nodes():
                 D.add_node(n)
                 new_n=Node(D,n)
-                for (k,v) in n.attr.iteritems():
-                    new_n.attr[k]=v
+                new_n.attr.update(n.attr)
             for e in self.edges():
                 (u,v)=e
                 D.add_edge(u,v)
                 D.add_edge(v,u)
                 uv=D.get_edge(u,v)
                 vu=D.get_edge(v,u)
-                for (k,v) in e.attr.iteritems():
-                    uv.attr[k]=v
-                    vu.attr[k]=v
+                uv.attr.update(e.attr)
+                uv.attr.update(e.attr)
+                vu.attr.update(e.attr)
             return D
         else:
             return self.copy()
@@ -947,8 +1051,8 @@ class AGraph(object):
         # so use tempfile version below
         return self.draw(format='dot',prog='nop') 
 
-    def string(self):
-        """Return string representation of graph in dot format.""" 
+    def to_string(self):
+        """Return a string containing the graph in dot format.""" 
         from tempfile import TemporaryFile
         fh = TemporaryFile()
         # Cover TemporaryFile wart: on 'nt' we need the file member
@@ -961,11 +1065,44 @@ class AGraph(object):
         fh.close()
         return data
 
+    def string(self):
+        """Return a string containing the graph in dot format.""" 
+        return self.to_string()
+
+    def from_string(self,string):
+        """ 
+        Load a graph from a string in dot format. Overwrites any existing graph.
+
+        To make a new graph from a string use
+        
+        >>> s='digraph {\n1 -> 2\n;}\n'
+        >>> A=AGraph()
+        >>> A.from_string(s)
+
+        or
+
+        >>> A=AGraph(string=s) # specify s is a string
+        >>> A=AGraph(s)  # s assumed to be a string during initialization
+
+        """
+        from tempfile import TemporaryFile
+        fh = TemporaryFile()
+        fh.write(string)
+        fh.seek(0)
+        # Cover TemporaryFile wart: on 'nt' we need the file member
+        if hasattr(fh, 'file'):
+            self.read(fh.file)
+        else:
+            self.read(fh)
+        fh.close()
+        return self
+
     def _get_prog(self,prog):
         # private: get path of graphviz program
         try:
             gvprogs=dict.fromkeys(\
-                ['neato','dot','twopi','circo','fdp','nop'])
+                ['neato','dot','twopi','circo','fdp','nop',
+                 'wc','acyclic','gvpr','gvcolor','ccomps','sccmap','tred'])
             p=gvprogs[prog]
         except KeyError:
             raise ValueError("prog %s is not one of: %s"%\
@@ -977,6 +1114,43 @@ class AGraph(object):
             raise ValueError("program %s not found in path"%prog) 
         return runprog
 
+    def _run_prog(self,prog='nop',args=''):
+        """Apply graphviz program to graph and return the result as a string.
+
+        >>> A=AGraph()
+        >>> s=A._run_prog() # uses 'nop' by default
+        >>> s=A._run_prog(prog='acyclic')
+
+        Use keyword args to add additional arguments to graphviz programs.
+
+        """
+        import os
+
+        runprog=self._get_prog(prog)
+        cmd=' '.join([runprog,args])
+        child_stdin,child_stdout,child_stderr=os.popen3(cmd, 'b')
+        # Use threading to avoid blocking
+        data = []
+        errors = []
+        threads = [PipeReader(data, child_stdout),
+                   PipeReader(errors, child_stderr)]
+        for t in threads:
+            t.start()
+
+        self.write(child_stdin)
+        child_stdin.close()
+
+        for t in threads:
+            t.join()
+
+        if data==[]:
+            raise IOError("".join(errors))
+# errors should be reported here somehow - is this the right way?
+#        if len(errors)>0:
+#            warnings.warn("".join(errors),UserWarning) 
+        return "".join(data)
+
+
     def layout(self,prog='neato',args='',fmt='dot'):
         """Assign positions to nodes in graph.
         
@@ -999,57 +1173,54 @@ class AGraph(object):
             sys.stderr.write(\
               "Warning: graph has %s nodes...layout may take a long time.\n"%\
               self.number_of_nodes())
-        runprog=self._get_prog(prog)
-        cmd=' '.join([runprog,args,"-T"+fmt])
-        child_stdin,child_stdout,child_stderr=os.popen3(cmd, 'b')
-        # Use threading to avoid blocking
-        # Use a temporary file because writing must be finished
-        # before starting to read
-        data = []
-        errors = []
-        threads = [PipeReader(data, child_stdout),
-                   PipeReader(errors, child_stderr)]
-        try:
-            for t in threads:
-                t.start()
-
-            self.write(child_stdin)
-            child_stdin.close()
-
-            for t in threads:
-                t.join()
-
-            if not data[0]:
-                raise IOError
-
-            # need to serialize writing and reading
-            # otherwise the internal state will be garbled
-            fh = TemporaryFile()
-            fh.write("".join(data))
-            fh.seek(0)
-            # Cover TemporaryFile wart: on 'nt' we need the file member
-            if hasattr(fh, 'file'):
-                self.read(fh.file)
-            else:
-                self.read(fh)
-            fh.close()
-        except:
-            if errors:
-                raise IOError("the graphviz layout with %s failed:\n%s "%
-                          (prog, "".join( errors )))
-            else:
-                raise IOError("the graphviz layout with %s failed."%prog)
 
+        data=self._run_prog(prog,' '.join([args,"-T",fmt]))
+        self.from_string(data)
         self.has_layout=True
         return
 
 
+    def tred(self,args='',copy=False):
+        """Transitive reduction of graph.  Modifies existing graph.
+        
+        To create a new graph use 
+
+        >>> A=AGraph()
+        >>> B=A.tred(copy=True)
+
+        See the graphviz "tred" program for details of the algorithm.
+        """
+        data=self._run_prog('tred',args)
+        if copy:
+            return self.__class__(string=data)
+        else:
+            return self.from_string(data)
+
+
+    def acyclic(self,args='',copy=False):
+        """Reverse sufficient edges in digraph to make graph acyclic.
+        Modifies existing graph.
+
+        To create a new graph use 
+
+        >>> A=AGraph()
+        >>> B=A.acyclic(copy=True)
+
+        See the graphviz "acyclic" program for details of the algorithm.
+        """
+        data=self._run_prog('acyclic',args)
+        if copy:
+            return self.__class__(string=data)
+        else:
+            return self.from_string(data)
+
+
     def draw(self,path=None,format=None,prog=None,args=''):
         """Output graph to path in specified format.
 
         An attempt will be made to guess the output format based
-        on the file extension.  If that fails the format keyword
-        will be used.
+        on the file extension of path.  If that fails the format keyword
+        will be used.  
         
         Formats::
 
@@ -1060,7 +1231,7 @@ class AGraph(object):
 
         If prog is not specified and the graph has positions
         (see layout()) then no additional graph positioning will
-        be preformed.
+        be performed.
                  
         Optional prog=['neato'|'dot'|'twopi'|'circo'|'fdp'|'nop']
         will use specified graphviz layout method.
@@ -1074,15 +1245,18 @@ class AGraph(object):
         # use dot to position, output png in 'file'
         >>> G.draw('file', format='png',prog='dot') 
 
-        Use keyword args to add additional arguments to graphviz programs.
+        # use keyword 'args' to pass additional arguments to graphviz
+        >>> G.draw('test.ps',prog='twopi',args='-Gepsilon=1')
 
         The layout might take a long time on large graphs.
 
         """
         import os
 
+        # try to guess format from extension
         if format is None and path is not None:
-            format=os.path.splitext(path)[1].lower()[1:]  # get extension
+            fh=self._get_fh(path,'w+b')
+            format=os.path.splitext(fh.name)[-1].lower()[1:] 
 
         if format is None or format=='':
             format = 'dot'
@@ -1114,46 +1288,20 @@ class AGraph(object):
               "Warning: graph has %s nodes...layout may take a long time.\n"%\
               self.number_of_nodes())
 
-        runprog=self._get_prog(prog)
         if prog=='nop': # nop takes no switches
-            cmd=prog
+            args=''
         else:
-            cmd=' '.join([prog,args,"-T"+format])
-        child_stdin,child_stdout,child_stderr=os.popen3(cmd, 'b')
+            args=' '.join([args,"-T"+format])
 
-        data = []
-        errors = []
-        threads = [PipeReader(data, child_stdout),
-                   PipeReader(errors, child_stderr)]
-
-        try:
-            for t in threads:
-                t.start()
-
-            self.write(child_stdin)
-            child_stdin.close()
-
-            for t in threads:
-                t.join()
-
-            if not data[0]:
-                raise IOError
-
-            if path is not None:
-                fh=self._get_fh(path,'w+b')
-                fh.write("".join(data))
-                fh.close()
-                d=None
-            else:
-                d="".join( data )
-                
-        except:
-            if errors:
-                raise IOError("the graphviz layout with %s failed:\n%s "%
-                              (prog, "".join( errors )) )
-            else:
-                raise IOError("the graphviz layout with %s failed."%prog)
+        data=self._run_prog(prog,args)
 
+        if path is not None:
+            fh=self._get_fh(path,'w+b')
+            fh.write("".join(data))
+            fh.close()
+            d=None
+        else:
+            d="".join( data )
         return d
 
     # some private helper functions
@@ -1285,12 +1433,13 @@ class Edge(tuple):
     >>> edge.attr['color']='red'
 
     """ 
-    def __new__(self,graph,source,target,key=None):
+    def __new__(self,graph,source,target,key=None,eh=None):
         s=Node(graph,source)
         t=Node(graph,target)
 
         try:
-            eh=gv.agedge(s.get_handle(),t.get_handle(),key,_Action.find)
+            if eh is None:
+                eh=gv.agedge(s.get_handle(),t.get_handle(),key,_Action.find)
         except KeyError:
             raise KeyError("edge %s-%s not in graph"%(source,target))
 
